#ifndef __ETHERNETIF_H__
#define __ETHERNETIF_H__

#include "lwip/err.h"
#include "lwip/netif.h"
#include "FreeRTOS.h"
#include "semphr.h"

/* Exported types ------------------------------------------------------------*/
/* Structure that include link thread parameters */
struct link_str {
  struct netif *netif;
  xSemaphoreHandle semaphore;
};

/* Exported functions ------------------------------------------------------- */
err_t ethernetif_init(struct netif *netif);      

void ethernetif_set_link(void const *argument);
void ethernetif_update_config(struct netif *netif);
void ethernetif_notify_conn_changed(struct netif *netif);

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define ENET_RXBD_NUM (6)
#define ENET_TXBD_NUM (5)
#define ENET_RXBUFF_SIZE (ENET_FRAME_MAX_FRAMELEN)
#define ENET_TXBUFF_SIZE (ENET_FRAME_MAX_FRAMELEN)
#define ENET_BuffSizeAlign(n) ENET_ALIGN(n, ENET_BUFF_ALIGNMENT)
#define ENET_DATA_LENGTH (1000)
#define ENET_TRANSMIT_DATA_NUM (20)
#define ENET_ALIGN(x, align) ((unsigned int)((x) + ((align)-1)) & (unsigned int)(~(unsigned int)((align)-1)))

#if defined(__GNUC__)
#ifndef __ALIGN_END
#define __ALIGN_END __attribute__((aligned(ENET_BUFF_ALIGNMENT)))
#endif
#ifndef __ALIGN_BEGIN
#define __ALIGN_BEGIN
#endif
#else
#ifndef __ALIGN_END
#define __ALIGN_END
#endif
#ifndef __ALIGN_BEGIN
#if defined(__CC_ARM)
#define __ALIGN_BEGIN __align(ENET_BUFF_ALIGNMENT)
#elif defined(__ICCARM__)
#define __ALIGN_BEGIN
#endif
#endif
#endif

#if FSL_FEATURE_ENET_SUPPORT_PTP
#define ENET_PTP_TXTS_RING_LEN (25)
#define ENET_PTP_RXTS_RING_LEN (25)
#define ENET_PTP_L2RING_LEN (15)
#define ENET_PTP_SYNC_MSG (0)
#define ENET_PTP_DELAYREQ_MSG (1)
#define ENET_PTP_ETHERNET_LAYER2_TEST (1)
#define ENET_IPV4_VERSION (4)
#endif

#define ENET_OK (0U)
#define ENET_ERROR (0xffU)
#define ENET_PHY_TIMEOUT (0xFFFFU)

#endif 
