/*
 * C
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLCOMM_UART.h"
#define   LLCOMM_BUFFERED_CONNECTION_IMPL LLCOMM_UART
#include "LLCOMM_BUFFERED_CONNECTION_impl.h"
#include "interrupts.h"
#include "fsl_uart.h"

/* Interrupt callback --------------------------------------------------------*/

void LLCOMM_UART_callback(LLCOMM_UART* llcomm)
{
	// retrieve LLCOM environment for given comIndex
	LLCOMM_BUFFERED_CONNECTION* env = &llcomm->header;
	UART_Type *base = ((LLCOMM_UART*)llcomm)->hal_uart;

	uint8_t leave = interrupt_enter();

	// check RX
	if (kUART_RxDataRegFullFlag & UART_GetStatusFlags(base))
	{
		// read data, clear the interrupt and send data to MicroEJ framework
		LLCOMM_BUFFERED_CONNECTION_dataReceived(env, base->D);
	}

	// check TX
	// warning: LLCOMM_BUFFERED_CONNECTION_transmitBufferReady() will disable the
	// TX interrupt but the a pending TX interrupt can occur after the disable
	// action -> use a boolean to ensure to not re-enter in function
	// LLCOMM_BUFFERED_CONNECTION_transmitBufferReady()
	if (llcomm->txInterruptEnabled && (kUART_TransmissionCompleteFlag & UART_GetStatusFlags(base)))
	{
		// transmit buffer empty -> notify MicroEJ framework (it will disable the TX interrupt)
		LLCOMM_BUFFERED_CONNECTION_transmitBufferReady(env);
	}

	interrupt_leave(leave);
}

/* API -----------------------------------------------------------------------*/

void LLCOMM_BUFFERED_CONNECTION_IMPL_initialize(LLCOMM_BUFFERED_CONNECTION* env)
{
	// pinout, clocks and interrupts have been already initialized 
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getPlatformId(LLCOMM_BUFFERED_CONNECTION* env)
{
	return ((LLCOMM_UART*)env)->platformId;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getName(LLCOMM_BUFFERED_CONNECTION* env)
{
	// this kind of connection cannot be open from a name (only from a platform port number)
	return 0;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_configureDevice(LLCOMM_BUFFERED_CONNECTION* env, int32_t baudrate, int32_t bitsperchar, int32_t stopbits, int32_t parity)
{
	UART_Type* handle = ((LLCOMM_UART*)env)->hal_uart;
	uart_config_t config;

	UART_GetDefaultConfig(&config);

	// get number of bits
	switch (bitsperchar)
	{
	default:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_5:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_6:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_7:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_9:
		// not available
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_BITSPERCHAR;
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_8:
		// only option available
		break;
	}

	// get number of stop bits
	switch (stopbits)
	{
	case LLCOMM_BUFFERED_CONNECTION_STOPBITS_1:
		config.stopBitCount = kUART_OneStopBit;
		break ;
	case LLCOMM_BUFFERED_CONNECTION_STOPBITS_2:
		config.stopBitCount = kUART_TwoStopBit;
		break ;
	default:
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_STOPBITS;
	}

	// get parity
	switch (parity)
	{
	case LLCOMM_BUFFERED_CONNECTION_PARITY_EVEN:
		config.parityMode = kUART_ParityEven;
		break ;
	case LLCOMM_BUFFERED_CONNECTION_PARITY_NO:
		config.parityMode = kUART_ParityDisabled;
		break ;
	case LLCOMM_BUFFERED_CONNECTION_PARITY_ODD:
		config.parityMode = kUART_ParityOdd;
		break ;
	default:
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_PARITY;
	}

	config.baudRate_Bps = baudrate;

	UART_Init(handle, &config, CLOCK_GetFreq(((LLCOMM_UART*)env)->hal_clock));

	return LLCOMM_BUFFERED_CONNECTION_SUCCESS;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_EnableRx(((LLCOMM_UART*)env)->hal_uart, true);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_EnableRx(((LLCOMM_UART*)env)->hal_uart, false);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_EnableInterrupts(((LLCOMM_UART*)env)->hal_uart, kUART_RxDataRegFullInterruptEnable);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_DisableInterrupts(((LLCOMM_UART*)env)->hal_uart, kUART_RxDataRegFullInterruptEnable);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_EnableTx(((LLCOMM_UART*)env)->hal_uart, true);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_EnableTx(((LLCOMM_UART*)env)->hal_uart, false);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->txInterruptEnabled = 1;
	UART_EnableInterrupts(((LLCOMM_UART*)env)->hal_uart, kUART_TransmissionCompleteInterruptEnable);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	UART_DisableInterrupts(((LLCOMM_UART*)env)->hal_uart, kUART_TransmissionCompleteInterruptEnable);
	((LLCOMM_UART*)env)->txInterruptEnabled = 0;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (int32_t)(((LLCOMM_UART*)env)->rxBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_UART_RX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (int32_t)(((LLCOMM_UART*)env)->txBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_UART_TX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_useTXHardwareFIFO(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_BUFFERED_CONNECTION_SUCCESS;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_txFifoFull(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_BUFFERED_CONNECTION_HARDWARE_FIFO_NOT_FULL;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_sendData(LLCOMM_BUFFERED_CONNECTION* env, int32_t data)
{
	uint8_t byteToSend = (uint8_t)(data & 0xFF);
	/* Transmit Data */
	UART_WriteBlocking(((LLCOMM_UART*)env)->hal_uart, &byteToSend, 1);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_isTransmissionComplete(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (kUART_TransmissionCompleteFlag & UART_GetStatusFlags(((LLCOMM_UART*)env)->hal_uart)) ? LLCOMM_BUFFERED_CONNECTION_TRANSMISSION_COMPLETE : LLCOMM_BUFFERED_CONNECTION_TRANSMISSION_TIMEOUT;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getNbProperties(LLCOMM_BUFFERED_CONNECTION* env)
{
	// This implementation does not provide Device properties
	return 0;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getProperty(LLCOMM_BUFFERED_CONNECTION* env, void* propertyName)
{
	// This implementation does not provide Device properties
	return 0;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getPropertyName(LLCOMM_BUFFERED_CONNECTION* env, int32_t propertyId)
{
	// This implementation does not provide Device properties
	while(1);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_released(LLCOMM_BUFFERED_CONNECTION* env)
{
	// This implementation does not support dynamic connections add/removal.
	// To enable dynamic connections, synchronization functions must be implemented.
	// See LLCOMM_impl.h
	while(1);
}

