/*
 * C
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include <stdint.h>
#include "LLCOMM_UART.h"
#include "LLCOMM_impl.h"
#include "microej.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "interrupts.h"
#include "uart.h"

#include "fsl_clock.h"
#include "fsl_gpio.h"
#include "fsl_port.h"

/* Methods declaration -------------------------------------------------------*/

void LLCOMM_UART_new(LLCOMM_BUFFERED_CONNECTION* env);

/* Enumerations --------------------------------------------------------------*/

/* Fields --------------------------------------------------------------------*/

/*
 * Array of all static LLCOMM UARTs.
 */
static LLCOMM_UART LLCOMM_UARTs[1];

static SemaphoreHandle_t sync_connections_sem;

/* Private functions ---------------------------------------------------------*/

/*
 * Initialize LLCOMM_UARTs array for a specific HAL UART instance and initialize
 * corresponding MicroEJ LLCOMM_UART.
 */
static void COMMS_initialize_llcomm_uart(int32_t index, UART_Type* hal_uart, uint8_t platform_id, clock_name_t hal_clock)
{
	// store HAL UART instance
	LLCOMM_UARTs[index].hal_uart = hal_uart;
	LLCOMM_UARTs[index].hal_clock = hal_clock;

	// store HAL UART ID
	LLCOMM_UARTs[index].platformId = platform_id;

	// create MicroEJ UART connection
	LLCOMM_UART_new(&LLCOMM_UARTs[index].header);

	// add MicroEJ UART connection into MicroEJ framework as generic connection
	LLCOMM_addConnection((LLCOMM_CONNECTION*)&LLCOMM_UARTs[index].header);
}

static void COMMS_initialize_interrupt(IRQn_Type interrupt)
{
	NVIC_SetPriority(interrupt, 0xF);
	NVIC_EnableIRQ(interrupt);
}

/* Callback ------------------------------------------------------------------*/

/* Interrupt functions -------------------------------------------------------*/

void UART0_RX_TX_IRQHandler(void)
{
	LLCOMM_UART_callback(&LLCOMM_UARTs[UART0_INDEX]);
}

/* Public functions ----------------------------------------------------------*/

/*
 * The semaphore must be created before the first call of LLCOMM_IMPL_syncConnectionsEnter()
 * which is called BEFORE LLCOMM_IMPL_initialize().
 */
void LLCOMM_stack_initialize(void)
{
	vSemaphoreCreateBinary(sync_connections_sem);
	xSemaphoreGive(sync_connections_sem);
}

void LLCOMM_IMPL_initialize(void)
{
	uart_config_t config;
	UART_GetDefaultConfig(&config);

	CLOCK_EnableClock(kCLOCK_PortA);
	PORT_SetPinMux(PORTA, 1U, kPORT_MuxAlt2);
	PORT_SetPinMux(PORTA, 2U, kPORT_MuxAlt2);
	COMMS_initialize_llcomm_uart(UART0_INDEX, UART0, 0, UART0_CLK);
	UART_Init(UART0, &config, CLOCK_GetFreq(UART0_CLK));
	COMMS_initialize_interrupt(UART0_RX_TX_IRQn);
}

void LLCOMM_IMPL_syncConnectionsEnter(void)
{
	xSemaphoreTake(sync_connections_sem, 0);
}

void LLCOMM_IMPL_syncConnectionsExit(void)
{
	if (interrupt_is_in() == MICROEJ_TRUE)
	{
		portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
		xSemaphoreGiveFromISR(sync_connections_sem, &xHigherPriorityTaskWoken);
		if(xHigherPriorityTaskWoken != pdFALSE )
		{
			// Force a context switch here.
			portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
		}
	}
	else
	{
		xSemaphoreGive(sync_connections_sem);
	}
}
