/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "hal_k65.h"
#include "adc_manager.h"

/* Enum ----------------------------------------------------------------------*/

/*
 * List of DAC IDs
 */
typedef enum
{
	HAL_DAC0_ID = 0,
	HAL_DAC1_ID = 1,
} hal_k65_dac_IDs;

/*
 * List of ADC IDs
 */
typedef enum
{
	HAL_ADC0_ID = 0,
	HAL_ADC1_ID,
} hal_k65_adc_IDs;

/*
 * List of ADC ranks
 */
typedef enum
{
	HAL_ADC0_SE6_ID = 0,
	HAL_ADC0_SE7_ID,
	HAL_ADC1_SE16_ID,
	HAL_ADC0_SE22_ID,
	HAL_ADC0_SE23_ID,
} hal_stm32_adc_ranks;


/* Fields --------------------------------------------------------------------*/


/* Public functions ----------------------------------------------------------*/

void hal_k65_initialize(void)
{
	// GPIO clocks
	CLOCK_EnableClock(kCLOCK_PortA);
	CLOCK_EnableClock(kCLOCK_PortB);
	CLOCK_EnableClock(kCLOCK_PortC);
	CLOCK_EnableClock(kCLOCK_PortD);
	CLOCK_EnableClock(kCLOCK_PortE);
}

int32_t dac_manager_impl_get_data(int32_t dac_ID, hal_k65_dac_data* dac_data)
{
	switch(dac_ID)
	{
	case HAL_DAC0_ID:
		dac_data->DACx = DAC0;
		break;
	case HAL_DAC1_ID:
		dac_data->DACx = DAC1;
		break;
	default:
		// invalid ID
		return HAL_GPIO_NOK;
	}
	return HAL_GPIO_OK;
}

int32_t adc_manager_impl_get_data(int32_t adc_ID, ADC_Type** adcHandle/*, DMA_HandleTypeDef** hdma_adc*/)
{
	switch (adc_ID)
	{
	case HAL_ADC0_ID:
		*adcHandle = ADC0;
		break;
	case HAL_ADC1_ID:
		*adcHandle = ADC1;
		break;

	default:
		// invalid ID
		return HAL_GPIO_NOK;
	}
	return HAL_GPIO_OK;
}

void adc_manager_impl_configure_ranks(int32_t adc_ID, ADC_Type* adcHandle)
{
	switch (adc_ID)
	{
	case HAL_ADC1_ID:
		adc_manager_configure_rank(adcHandle, 1, 16);	// ADC1_SE16
		break;
	}
}

int32_t hal_k65_get_gpio_data(int32_t port, int32_t pin,
		hal_k65_gpio_data* gpio) {
	gpio->GPIOx = NULL;
	gpio->PORTx = NULL;
	switch (port) {
	case HAL_K65_GPIO_PORT_MCU: {
		/*
		 * MCU virtual port: convert port and pin using real MCU port and pin
		 *
		 * MCU port = pin / 32
		 * MCU pin = pin % 32
		 */

		if (pin < 0)
		{
			return HAL_GPIO_NOK;	// invalid pin
		}
		return hal_k65_get_gpio_data(pin >> 5, pin & 0x1f, gpio);
		break;
	}

	case LLHAL_GPIO_PORT_A: {
		/*
		 * GPIOA check and initialization
		 */
		switch (pin) {
		case 0:
		case 3:
		case 9:
		case 11:
		case 18:
		case 19:
		case 28:
		case 29:
			// only digital pin
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}

		gpio->GPIOx = GPIOA;
		gpio->PORTx = PORTA;
		gpio->GPIO_Pin = pin;
		break;
	}

	case LLHAL_GPIO_PORT_B: {
		/*
		 * GPIOB check and initialization
		 */
		switch (pin) {
		case 19:
			// only digital pin
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}

		gpio->GPIOx = GPIOB;
		gpio->PORTx = PORTB;
		gpio->GPIO_Pin = pin;
		break;
	}

	case LLHAL_GPIO_PORT_C: {
		/*
		 * GPIOC check and initialization
		 */
		switch (pin) {
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}

		gpio->GPIOx = GPIOC;
		gpio->PORTx = PORTC;
		gpio->GPIO_Pin = pin;
		break;
	}

	case LLHAL_GPIO_PORT_D: {
		/*
		 * GPIOD check and initialization
		 */
		switch (pin) {
		case 0:
		case 10:
		case 11:
		case 12:
		case 13:
		case 14:
			// only digital pin
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}

		gpio->GPIOx = GPIOD;
		gpio->PORTx = PORTD;
		gpio->GPIO_Pin = pin;
		break;
	}

	case LLHAL_GPIO_PORT_E: {
		/*
		 * GPIOB check and initialization
		 */
		switch (pin) {
		case 18:
			//ADC0_SE6A
//			gpio->adc_rank = HAL_ADC0_SE6_ID;
			break;
		case 19:
			//ADC0_SE7A
//			gpio->adc_rank = HAL_ADC0_SE7_ID;
			break;
		case 6:
		case 7:
		case 8:
		case 9:
		case 10:
		case 11:
		case 12:
		case 27:
			// only digital pin
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}

		gpio->GPIOx = GPIOE;
		gpio->PORTx = PORTE;
		gpio->GPIO_Pin = pin;
		break;
	}
	/* See the schematics for details here */
	case LLHAL_GPIO_PORT_ELEVA: {
		switch (pin) {
		case 7:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 19;
			break;
		case 8:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 18;
			break;
		case 11:
			gpio->GPIOx = GPIOA;
			gpio->PORTx = PORTA;
			gpio->GPIO_Pin = 9;
			break;
		case 21:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 6;
			break;
		case 22:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 12;
			break;
		case 23:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 11;
			break;
		case 24:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 7;
			break;
		case 25:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 10;
			break;
		case 32:
			//DAC0_OUT
			gpio->dac_data.DACx = DAC0;
			//ADC0_SE23
//			gpio->adc_rank = HAL_ADC0_SE23_ID;
			break;
		case 35:
			gpio->GPIOx = GPIOA;
			gpio->PORTx = PORTA;
			gpio->GPIO_Pin = 29;
			break;
		case 58:
		case 61:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 9;
			break;
		case 59:
		case 60:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 8;
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}
		break;
	}

	case LLHAL_GPIO_PORT_ELEVB: {
		switch (pin) {
		case 23:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 10;
			break;
		case 32:
			//DAC1_OUT
			gpio->dac_data.DACx = DAC1;
			break;
		case 35:
			gpio->GPIOx = GPIOA;
			gpio->PORTx = PORTA;
			gpio->GPIO_Pin = 28;
			break;
		case 44:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 14;
			break;
		case 45:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 13;
			break;
		case 46:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 11;
			break;
		case 47:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 15;
			break;
		case 48:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 12;
			break;
		case 55:
		case 56:
			gpio->GPIOx = GPIOE;
			gpio->PORTx = PORTE;
			gpio->GPIO_Pin = 27;
			break;
		case 63:
			gpio->GPIOx = GPIOD;
			gpio->PORTx = PORTD;
			gpio->GPIO_Pin = 0;
			break;
		case 72:
			gpio->GPIOx = GPIOB;
			gpio->PORTx = PORTB;
			gpio->GPIO_Pin = 19;
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}
		break;
	}

	case LLHAL_GPIO_PORT_ELEVC: {
		switch (pin) {
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}
		break;
	}

	case LLHAL_GPIO_PORT_ELEVD: {
		switch (pin) {
		case 30:
			//ADC1_SE16
			gpio->adc_rank = HAL_ADC1_SE16_ID;
			break;
		default:
			// invalid pin
			return HAL_GPIO_NOK;
		}
		break;
	}

	default:
		// invalid port
		return HAL_GPIO_NOK;
	}
	return HAL_GPIO_OK;
}

/* EOF -----------------------------------------------------------------------*/
