/*
 * C
 *
 * Copyright 2013-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "microej.h"
#include "interrupts.h"
#include "LLMJVM.h"
#include "MK65F18.h"
#include "fsl_gpio.h"
#include "fsl_port.h"
#include "buttons_manager.h"

/* Global --------------------------------------------------------------------*/

static GPIO_Type* BUTTON_GPIO[2] = {GPIOA, GPIOA};
static PORT_Type* BUTTON_PORT[2] = {PORTA, PORTA};
static const uint16_t BUTTON_PIN[2] = {4U, 10U};
static const IRQn_Type BUTTON_IRQn[2] = {PORTA_IRQn, PORTA_IRQn};
static const Button_TypeDef BUTTON_ID[2] = {SW2, SW3};
static const uint8_t BUTTON_REVERSE[2] = {MICROEJ_TRUE, MICROEJ_TRUE};

/* Private API ---------------------------------------------------------------*/

static void BUTTONS_MANAGER_event(Button_TypeDef Button)
{
	uint8_t button_pressed;
        uint32_t readValue = GPIO_ReadPinInput(BUTTON_GPIO[Button], BUTTON_PIN[Button]);

        if (readValue == 1U)
	{
		// GPIO == 1

		if (BUTTON_REVERSE[Button] == MICROEJ_TRUE)
		{
			// GPIO == 1 means "released"
			button_pressed = MICROEJ_FALSE;
		}
		else
		{
			// GPIO == 1 means "pressed"
			button_pressed = MICROEJ_TRUE;
		}
	}
	else
	{
		// GPIO == 0

		if (BUTTON_REVERSE[Button] == MICROEJ_TRUE)
		{
			// GPIO == 0 means "pressed"
			button_pressed = MICROEJ_TRUE;
		}
		else
		{
			// GPIO == 0 means "released"
			button_pressed = MICROEJ_FALSE;
		}
	}

	if (button_pressed == MICROEJ_TRUE)
	{
		BUTTONS_HELPER_pressed(BUTTON_ID[Button]);
		LLMJVM_dump();
	}
	else
	{
		BUTTONS_HELPER_released(BUTTON_ID[Button]);
	}
}

static void BUTTONS_MANAGER_init(Button_TypeDef Button)
{
  gpio_pin_config_t sw_config = {
      kGPIO_DigitalInput, 0,
  };

  /* Enable the BUTTON Clock */
  CLOCK_EnableClock(kCLOCK_PortA);

  /* Configure Button pin as input */
  port_pin_config_t config = {0};
  config.pullSelect = kPORT_PullUp;
  config.mux = kPORT_MuxAsGpio;
  PORT_SetPinConfig(BUTTON_PORT[Button], BUTTON_PIN[Button], &config);

  PORT_SetPinInterruptConfig(BUTTON_PORT[Button], BUTTON_PIN[Button], kPORT_InterruptEitherEdge);
  GPIO_PinInit(BUTTON_GPIO[Button], BUTTON_PIN[Button], &sw_config);
  
  NVIC_SetPriority (BUTTON_IRQn[Button], 0xf);
  EnableIRQ(BUTTON_IRQn[Button]);
}

/* API -----------------------------------------------------------------------*/

void BUTTONS_MANAGER_interrupt(Button_TypeDef Button)
{
  uint32_t tmpFlags = GPIO_GetPinsInterruptFlags(BUTTON_GPIO[Button]);
  
  if((tmpFlags & (1U << BUTTON_PIN[Button])) != 0)
  {
    uint8_t leave = interrupt_enter();
    BUTTONS_MANAGER_event(Button);
    interrupt_leave(leave);
    GPIO_ClearPinsInterruptFlags(BUTTON_GPIO[Button], 1U << BUTTON_PIN[Button]);
  }
}

void BUTTONS_MANAGER_enable_interrupts(void)
{
  EnableIRQ(PORTA_IRQn);
}

void BUTTONS_MANAGER_disable_interrupts(void)
{
  DisableIRQ(PORTA_IRQn);
}

void BUTTONS_MANAGER_initialize(void)
{
	BUTTONS_HELPER_initialize();
	BUTTONS_MANAGER_init(SW2);
	BUTTONS_MANAGER_init(SW3);
}
