/*
 * C
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>

#include "microej.h"
#include "fsl_common.h"
#include "fsl_clock.h"
#include "fsl_gpio.h"
#include "fsl_port.h"
#include "fsl_adc16.h"

#include "touch_helper.h"

/* Defines -------------------------------------------------------------------*/
#define TOUCH_SCREEN_WIDTH  (320U)
#define TOUCH_SCREEN_HEIGHT (240U)

/* Empiric calibration values */
#define TOUCH_LOW_X   (200U)
#define TOUCH_HIGH_X (3800U)
#define TOUCH_LOW_Y   (400U)
#define TOUCH_HIGH_Y (3700U)

#define NXP_TOUCH_FIX

/* Global --------------------------------------------------------------------*/
adc16_config_t adcConfig;

/* Private API ---------------------------------------------------------------*/

uint8_t TOUCH_MANAGER_ReadADCFromY(uint32_t *coord)
{
  uint8_t isValidRead = MICROEJ_FALSE;
  adc16_channel_config_t adcChannelConfig;
  
  if (!(ADC0->SC2 & ADC_SC2_ADACT_MASK))
  {
    ADC16_Init(ADC0, &adcConfig);
    ADC16_SetHardwareAverage(ADC0, kADC16_HardwareAverageCount4);
    ADC16_EnableHardwareTrigger(ADC0, false);
    
    adcChannelConfig.channelNumber = 17;
    adcChannelConfig.enableInterruptOnConversionCompleted = false;
    adcChannelConfig.enableDifferentialConversion = false;
    
    ADC16_SetChannelConfig(ADC0, 0, &adcChannelConfig);
    
    while (0U == (kADC16_ChannelConversionDoneFlag &
                      ADC16_GetChannelStatusFlags(ADC0, 0)));
    
    *coord = ADC16_GetChannelConversionValue(ADC0, 0);
    
    if (*coord >= TOUCH_LOW_Y)
    {
      isValidRead = MICROEJ_TRUE;
    }
  }
  
  return isValidRead;
}

uint8_t TOUCH_MANAGER_ReadADCFromX(uint32_t *coord)
{
  uint8_t isValidRead = MICROEJ_FALSE;
  adc16_channel_config_t adcChannelConfig;
  
  if (!(ADC0->SC2 & ADC_SC2_ADACT_MASK))
  { 
    ADC16_Init(ADC0, &adcConfig);
    ADC16_SetHardwareAverage(ADC0, kADC16_HardwareAverageCount4);
    ADC16_EnableHardwareTrigger(ADC0, false);
    
    adcChannelConfig.channelNumber = 18;
    adcChannelConfig.enableInterruptOnConversionCompleted = false;
    adcChannelConfig.enableDifferentialConversion = false;
    
    ADC16_SetChannelConfig(ADC0, 0, &adcChannelConfig);
    
    while (0U == (kADC16_ChannelConversionDoneFlag &
                      ADC16_GetChannelStatusFlags(ADC0, 0)));
    
    *coord = ADC16_GetChannelConversionValue(ADC0, 0);
    
    if (*coord >= TOUCH_LOW_X)
    {
      isValidRead = MICROEJ_TRUE;
    }
  }
  
  return isValidRead;
}

static uint8_t TOUCH_MANAGER_GetState(uint16_t* x, uint16_t* y)
{
  uint8_t isValidConversion = MICROEJ_FALSE;
  
  uint32_t tempx = 0;
  uint32_t tempy = 0;
  
  gpio_pin_config_t outputHigh = {
      kGPIO_DigitalOutput, 1
  };
  gpio_pin_config_t input = {
      kGPIO_DigitalInput, 0
  };
  gpio_pin_config_t outputLow = {
      kGPIO_DigitalOutput, 0
  };
  
  // Switch Y- for ADC
  PORTE->PCR[24]  = PORT_PCR_MUX(kPORT_PinDisabledOrAnalog);

  // Set X- high
  GPIO_PinInit(GPIOC, 3U, &outputHigh);

#ifdef NXP_TOUCH_FIX
  // Set Y+ high-Z
  GPIO_PinInit(GPIOE, 28U, &input);
#else
  // Set Y+ high-Z
  GPIO_PinInit(GPIOA, 6U, &input);
#endif
  // Read the value of Y-
  if (MICROEJ_TRUE == TOUCH_MANAGER_ReadADCFromY(&tempy))
  {
	// If touched:
    // Disable ADC for Y-
    PORTE->PCR[24]  = PORT_PCR_MUX(kPORT_MuxAsGpio);
    // Reset pins
#ifdef NXP_TOUCH_FIX
    GPIO_PinInit(GPIOE, 28U, &outputLow);
#else
    GPIO_PinInit(GPIOA, 6U, &outputLow);
#endif
    GPIO_PinInit(GPIOC, 3U, &outputLow);
    GPIO_PinInit(GPIOE, 24U, &outputLow);
    GPIO_PinInit(GPIOE, 25U, &outputLow);
    // Switch X+ for ADC
    PORTE->PCR[25]  = PORT_PCR_MUX(kPORT_PinDisabledOrAnalog);
    // Set Y+ high
#ifdef NXP_TOUCH_FIX
    GPIO_PinInit(GPIOE, 28U, &outputHigh);
#else
    GPIO_PinInit(GPIOA, 6U, &outputHigh);
#endif
    // Set X- high-Z
    GPIO_PinInit(GPIOC, 3U, &input);
    // Read the value of X+
    isValidConversion = TOUCH_MANAGER_ReadADCFromX(&tempx);

    // Disable ADC for X+
    PORTE->PCR[25]  = PORT_PCR_MUX(kPORT_MuxAsGpio);
  }
    
    // Reset pins
#ifdef NXP_TOUCH_FIX
    GPIO_PinInit(GPIOE, 28U, &outputLow);
#else
    GPIO_PinInit(GPIOA, 6U, &outputLow);
#endif
    GPIO_PinInit(GPIOC, 3U, &outputLow);
    GPIO_PinInit(GPIOE, 24U, &outputLow);
    GPIO_PinInit(GPIOE, 25U, &outputLow);
  
    if (MICROEJ_TRUE == isValidConversion)
    {
      *x = TOUCH_SCREEN_WIDTH - (((tempx - TOUCH_LOW_X)*TOUCH_SCREEN_WIDTH)/(TOUCH_HIGH_X-TOUCH_LOW_X));
      *y = (((tempy - TOUCH_LOW_Y)*TOUCH_SCREEN_HEIGHT)/(TOUCH_HIGH_Y-TOUCH_LOW_Y));
    }
  return isValidConversion;
}


/* API -----------------------------------------------------------------------*/

void TOUCH_MANAGER_initialize(void)
{
  gpio_pin_config_t sw_config = {
      kGPIO_DigitalOutput, 0,
  };
  
  /* Enable the needed clocks */
  CLOCK_EnableClock(kCLOCK_PortA);
  CLOCK_EnableClock(kCLOCK_PortC);
  CLOCK_EnableClock(kCLOCK_PortE);

#ifdef NXP_TOUCH_FIX
  PORTE->PCR[28]  = PORT_PCR_MUX(kPORT_MuxAsGpio); // Y+
#else
  PORTA->PCR[6]  = PORT_PCR_MUX(kPORT_MuxAsGpio); // Y+
#endif
  PORTC->PCR[3]  = PORT_PCR_MUX(kPORT_MuxAsGpio); // X-
  PORTE->PCR[24]  = PORT_PCR_MUX(kPORT_MuxAsGpio); // Y-
  PORTE->PCR[25]  = PORT_PCR_MUX(kPORT_MuxAsGpio); // X+

#ifdef NXP_TOUCH_FIX
  GPIO_PinInit(GPIOE, 28U, &sw_config);
#else
  GPIO_PinInit(GPIOA, 6U, &sw_config);
#endif
  GPIO_PinInit(GPIOC, 3U, &sw_config);
  GPIO_PinInit(GPIOE, 24U, &sw_config);
  GPIO_PinInit(GPIOE, 25U, &sw_config);
  
  adcConfig.referenceVoltageSource = kADC16_ReferenceVoltageSourceVref;
  adcConfig.clockSource = kADC16_ClockSourceAsynchronousClock;
  adcConfig.enableAsynchronousClock = true;
  adcConfig.clockDivider = kADC16_ClockDivider8;
  adcConfig.resolution = kADC16_ResolutionSE12Bit;
  adcConfig.longSampleMode = kADC16_LongSampleCycle24;
  adcConfig.enableHighSpeed = false;
  adcConfig.enableLowPower = false;
  adcConfig.enableContinuousConversion = false;
    
  ADC16_Init(ADC0, &adcConfig);
  ADC16_SetHardwareAverage(ADC0, kADC16_HardwareAverageCount4);
  ADC16_EnableHardwareTrigger(ADC0, false);
  
#if defined(FSL_FEATURE_ADC16_HAS_CALIBRATION) && FSL_FEATURE_ADC16_HAS_CALIBRATION
  ADC16_DoAutoCalibration(ADC0);
#endif /* FSL_FEATURE_ADC16_HAS_CALIBRATION */
}

void TOUCH_MANAGER_update(void)
{
	uint16_t x,y;

	if (TOUCH_MANAGER_GetState(&x, &y) == MICROEJ_TRUE)
	{
		// here, pen is down for sure
                TOUCH_HELPER_pressed(x, y);
	}
	else
	{
		// here, pen is up for sure
		TOUCH_HELPER_released();
	}
}
