/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "MK65F18.h"
#include "sram_driver.h"
#include "fsl_common.h"
#include "fsl_clock.h"
#include "fsl_flexbus.h"
#include "fsl_port.h"
#include "fsl_sdram.h"
#include "stdint.h"

void twr_sdram_clock_init(void)
{
    /* Enable the clock to the FlexBus module */
    CLOCK_EnableClock(kCLOCK_Flexbus0);
    SIM->SCGC7 |= SIM_SCGC7_FLEXBUS_MASK;

    /* Enable the FB_CLKOUT function on PTA6 (alt5 function) */
    CLOCK_SetClkOutClock(0);
    PORTA->PCR[6] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK; // enable high drive strength to support high toggle rate
}

void twr_sdram_pins_init(void)
{
    /* Initialize the SDRAMC pins */

    // BS3
    PORTB->PCR[20] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[31]
    PORTB->PCR[21] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[30]
    PORTB->PCR[22] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[29]
    PORTB->PCR[23] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[28]
    PORTC->PCR[12] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[27]
    PORTC->PCR[13] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[26]
    PORTC->PCR[14] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[25]
    PORTC->PCR[15] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[24]

    // BS2
    PORTB->PCR[6]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[23]
    PORTB->PCR[7]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[22]
    PORTB->PCR[8]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[21]
    PORTB->PCR[9]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[20]
    PORTB->PCR[10] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[19]
    PORTB->PCR[11] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[18]
    PORTB->PCR[16] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[17]
    PORTB->PCR[17] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[16]

    //BS1
    PORTA->PCR[24] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[15]
    PORTA->PCR[25] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[14]
    PORTA->PCR[26] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[13]
    PORTA->PCR[27] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[12]
    PORTA->PCR[30] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[11]
    PORTA->PCR[31] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[10]
    PORTB->PCR[12] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[9]
    PORTB->PCR[13] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[8]

    //BS0
    PORTB->PCR[14] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[7]
    PORTB->PCR[15] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[6]
    PORTC->PCR[24] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[5]
    PORTC->PCR[25] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[4]
    PORTC->PCR[26] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[3]
    PORTC->PCR[27] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[2]
    PORTC->PCR[28] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[1]
    PORTC->PCR[29] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_D[0]

    //Configure SDRAM Address  GPIOs
    //1. set pin mux as SDRAM_xx function
    //2. high drive strength set
    PORTD->PCR[5]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[9]
    PORTD->PCR[4]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[10]
    PORTD->PCR[3]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[11]
    PORTD->PCR[2]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[12]
    PORTC->PCR[10] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[13]
    PORTC->PCR[9]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[14]
    PORTC->PCR[8]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[15]
    PORTC->PCR[7]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[16]
    PORTC->PCR[6]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[17]
    PORTC->PCR[5]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[18]
    PORTC->PCR[4]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[19]
    PORTC->PCR[2]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[20]
    PORTC->PCR[1]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[21]
    PORTC->PCR[0]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[22]
    PORTB->PCR[18] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;           //  SDRAM_A[23]

    //Config control signals
    // Enable the SDRAMC_CLKOUT function on PTC3 (alt5 function)
    // select Flexbus clock
    CLOCK_SetClkOutClock(0);
    PORTC->PCR[3]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_CLK

    PORTD->PCR[7]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_CKE
    PORTB->PCR[0]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_CAS_B
    PORTB->PCR[1]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_RAS_B
    PORTB->PCR[3]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_CS0_B
    PORTB->PCR[4]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_CS1_B
    PORTB->PCR[2]  = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_WE
    PORTC->PCR[17] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_DQM3
    PORTC->PCR[16] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_DQM2
    PORTC->PCR[18] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_DQM1
    PORTC->PCR[19] = PORT_PCR_MUX(kPORT_MuxAlt5)|PORT_PCR_DSE_MASK;          // SDRAM_DQM0
}

void SRAM_initialize(void)
{
    /* Ungate the port clock */
    CLOCK_EnableClock(kCLOCK_PortA);
    CLOCK_EnableClock(kCLOCK_PortB);
    CLOCK_EnableClock(kCLOCK_PortC);
    CLOCK_EnableClock(kCLOCK_PortD);

    twr_sdram_clock_init();

    SIM->SCGC7 |= SIM_SCGC7_SDRAMC_MASK | SIM_SCGC7_FLEXBUS_MASK;
    // fb_byte select and sdram_dqm functionality enabled, otherwise, no sdram_dqm output
    FB->CSPMCR = FB_CSPMCR_GROUP2(kFLEXBUS_MultiplexGroup2_FB_BE_31_24)
                |FB_CSPMCR_GROUP3(kFLEXBUS_MultiplexGroup3_FB_BE_23_16)
                |FB_CSPMCR_GROUP4(kFLEXBUS_MultiplexGroup4_FB_BE_15_8)
                |FB_CSPMCR_GROUP5(kFLEXBUS_MultiplexGroup5_FB_BE_7_0);
    // Disable flexbus clock after CSPMCR write to avoid unexpected behavior
    SIM->SCGC7 &= ~SIM_SCGC7_FLEXBUS_MASK;

    twr_sdram_pins_init();

    SDRAM_Init(SDRAM, CLOCK_GetFlexBusClkFreq());
}
